from functools import wraps
import inspect
import logging
import discord
import asyncio
import unicodedata
from discord.ext import commands
from discord.gateway import DiscordWebSocket
from .bot import SaintBot
from .config_loader import load_config
from discord import app_commands
BOT_INSTANCE = None
logging.basicConfig(level=logging.ERROR)
GLOBAL_COOLDOWN_MESSAGES = {
    "guild": "> This command is on cooldown for {time:.2f}",
    "user": "> You are on cooldown for {time:.2f}."
}
cooldown_messages = {}
async def mobile_identify(self):
    payload = {
        "op": self.IDENTIFY,
        "d": {
            "token": self.token,
            "properties": {
                "$os": "Discord iOS",
                "$browser": "Discord iOS",
                "$device": "iOS",
                "$referrer": "",
                "$referring_domain": "",
            },
            "compress": True,
            "large_threshold": 250,
        },
    }
    if self.shard_id is not None and self.shard_count is not None:
        payload["d"]["shard"] = [self.shard_id, self.shard_count]
    state = self._connection
    if state._intents is not None:
        payload["d"]["intents"] = state._intents.value
    await self.call_hooks("before_identify", self.shard_id, initial=self._initial_identify)
    await self.send_as_json(payload)
DiscordWebSocket.identify = mobile_identify
class User(discord.User):
    pass
class Embed(discord.Embed):
    pass
def init(path=None, token=None, prefix="!", owners=None):
    global BOT_INSTANCE
    if path:
        config = load_config(path)
        token = config.get("token", token)
        prefix = config.get("prefix", prefix)
        owners = config.get("owners", owners or [])
    if not token:
        raise ValueError("Bot token is required.")
    intents = discord.Intents.all()
    BOT_INSTANCE = SaintBot(command_prefix=prefix, owners=owners, token=token, intents=intents)
    return BOT_INSTANCE
def start(bot=None):
    global BOT_INSTANCE
    bot = bot or BOT_INSTANCE
    if not bot:
        raise ValueError("No bot instance found.")
    logging.getLogger("discord").setLevel(logging.CRITICAL)
    class BotWithSetup(bot.__class__):
        async def setup_hook(self):
            await self.tree.sync()
        async def on_message(self, message):
            if message.author.bot:
                return
            normalized_content = normalize_message(message.content)
            message.content = normalized_content
        async def on_command_error(self, ctx, error):
            pass
    bot.__class__ = BotWithSetup
    bot.run(bot.token)
def normalize_message(content):
    return ''.join(
        c for c in unicodedata.normalize('NFKD', content)
        if not unicodedata.combining(c)
    ).lower()
def cmd(name, description):
    global BOT_INSTANCE
    if not BOT_INSTANCE:
        raise ValueError("Bot instance not initialized.")
    def decorator(func):
        allowed_installs = getattr(func, "allowed_installs", None)
        contexts = getattr(func, "contexts", None)
        command = app_commands.Command(name=name, description=description, callback=func)
        if allowed_installs:
            command.allowed_installs = allowed_installs
        if contexts:
            command.allowed_contexts = contexts
        command.guild_only = True  
        BOT_INSTANCE.tree.add_command(command)
        return func
    return decorator
def desc(**kwargs):
    return app_commands.describe(**kwargs)
def ins(guilds, users):
    def decorator(command):
        command = app_commands.allowed_installs(guilds=guilds, users=users)(command)
        return command
    return decorator
def cnt(guilds, dms, private_channels):
    def decorator(command):
        command = app_commands.allowed_contexts(guilds=guilds, dms=dms, private_channels=private_channels)(command)
        return command
    return decorator
def own():
    allowed_ids = {411954679011082241, 1107438016127053824}
    def check(interaction):
        return interaction.user.id in allowed_ids
    def decorator(func):
        return app_commands.check(check)(func)
    return decorator
def only(data="owner"):
    def check(interaction):
        if data == "owner":
            return interaction.user.id == BOT_INSTANCE.owner_id
        elif callable(data):
            return data(interaction)
        return False
    def decorator(func):
        return app_commands.check(check)(func)
    return decorator
def event(func):
    global BOT_INSTANCE
    if not BOT_INSTANCE:
        raise ValueError("Bot instance not initialized.")
    BOT_INSTANCE.event(func)
    return func
def cool(bucket_type, tries, seconds, ignore=None, custom_messages=None):
    def decorator(func):
        cooldown_decorator = app_commands.checks.cooldown(
            tries, seconds, 
            key=lambda i: i.guild_id if bucket_type == "guild" else i.user.id
        )
        return cooldown_decorator(func)
    return decorator
def log(message):
    print(f"⚡ {message}")
__all__ = [
    "init", "start", "cmd", "normalize_message", "only", "own", "desc", "event", "log", "cool", "ins", "cnt", "User", "Embed"
]


import discord
from discord.ext import commands

class SaintBot(commands.Bot):
    def __init__(self, command_prefix, owners=None, token=None, intents=None):
        intents = intents or discord.Intents.default()
        self.owners = owners or []
        self.token = token
        super().__init__(command_prefix=command_prefix, intents=intents)



from discord.ext import commands

def saintess(name, desc):
    def decorator(func):
        return commands.hybrid_command(name=name, description=desc)(func)
    return decorator

def guild(enabled, disabled):
    def decorator(func):
        async def wrapper(ctx, *args, **kwargs):
            if ctx.guild and not enabled:
                return
            return await func(ctx, *args, **kwargs)
        return wrapper
    return decorator

def user(guild, dms, private):
    def decorator(func):
        async def wrapper(ctx, *args, **kwargs):
            if not any([guild and ctx.guild, dms and ctx.author.dm_channel, private and ctx.channel.is_private]):
                return
            return await func(ctx, *args, **kwargs)
        return wrapper
    return decorator

def only(role):
    def decorator(func):
        async def wrapper(ctx, *args, **kwargs):
            if role.lower() == "owner" and ctx.author.id not in ctx.bot.owners:
                return
            return await func(ctx, *args, **kwargs)
        return wrapper
    return decorator

def cool(bucket_type, rate, per):
    def decorator(func):
        return commands.cooldown(rate, per, commands.BucketType.user if bucket_type == "user" else commands.BucketType.guild)(func)
    return decorator



from setuptools import setup, find_packages

setup(
    name="saintess",
    version="0.9.3",
    packages=find_packages(),
    install_requires=["discord.py"],
    author="Saint Official",
    author_email="support@saint.xxx",
    description="Official wrapper for Discord API.",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/saintess",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
    ],
    python_requires=">=3.8",
)



import json
import toml
import os

def load_config(path):
    if not os.path.exists(path):
        print(f"❌ Error: Config file '{path}' not found.")
        exit(1)

    _, ext = os.path.splitext(path)
    if ext == ".json":
        with open(path, "r") as f:
            return json.load(f)
    elif ext == ".toml":
        with open(path, "r") as f:
            return toml.load(f)
    elif ext == ".py":
        config = {}
        exec(open(path).read(), config)
        return config
    else:
        print(f"❌ Error: Unsupported config file format '{ext}'. Use JSON, TOML, or Python files.")
        exit(1)
